@@ -234,3 +234,53 @@ class ScaLAPACK4py:
     self.pdgemr2d(m, n, src_data, 1, 1, src_desc, dest_data, 1, 1, dest_desc, common_blacs_ctx)
     self.close_blacs_context(gatherer_blacs_ctx)
 
+  def gather_numpy(self, descr, data, shape):
+    '''
+    Gather BLACS array to the process in 0-th row and column in a BLACS process grid
+    
+    :param descr: Pointer to BLACS descriptor or NULL if data not distributed
+    :param data:  Pointer to the array or its part in the current process
+    :param shape: Expected shape of the total array
+    :type descr:  ctypes.POINTER(c_int)
+    :type data:   ctypes.POINTER
+    :type shape:  (int, int)
+    :return:      Gathered array for the process (0,0), None for other processes
+    '''
+    if not descr:
+      return np.ctypeslib.as_array(data, shape=shape)
+    descr = self.wrap_blacs_desc(descr)
+    data = np.ctypeslib.as_array(data, shape=(descr.locrow, descr.loccol)).T
+    res = np.zeros(shape, dtype=np.float64).T if (descr.myrow==0 and descr.mycol==0) else None
+    self.basic_gather(descr, data, res)
+    return res
+
+  def scatter_numpy(self, src, descr, dst):
+    '''
+    Scatters numpy array over BLACS descriptor
+    :param src:   Source array on a root process, must be None on non-root processes
+    :param descr: Pointer to BLACS descriptor of the destination (`dst`) or NULL if data not distributed
+    :param dst:   Pointer to the destination BLACS array
+    :type src:    numpy.ndarray
+    :type descr:  ctypes.POINTER(c_int)
+    :type data:   ctypes.POINTER
+    '''
+    if not descr:
+      dst = np.ctypeslib.as_array(dst, shape=src.shape)
+      dst[:,:] = src
+      return
+
+    descr = self.wrap_blacs_desc(descr)
+    dst = np.ctypeslib.as_array(dst, shape=(descr.locrow, descr.loccol)).T
+    self.basic_scatter(src, descr, dst)
+
+  def is_root(self, descr):
+    '''
+    Returns True if descr is NULL or if the current process grid index is (0,0). False otherwise.
+    :param descr: Pointer to BLACS descriptor or NULL if data not distributed
+    :type descr:  ctypes.POINTER(c_int)
+    :return:      True if descr is NULL or if the current process grid index is (0,0). False otherwise.
+    '''
+    if not descr:
+      return True
+    descr = self.wrap_blacs_desc(descr)
+    return (descr.myrow==0 and descr.mycol==0)
